/* vim: set ai et sw=4 ts=4: */

#ifdef HAVE_CONFIG_H
#include <config.h>
#endif

#include <gnome.h>
#include <gst/gst.h>
#include <gst/gconf/gconf.h>

#include "applet.h"

/* callbacks */
static void engine_meet_end_of_stream (GstElement *in, gpointer data);

Engine*
engine_new (void)
{
    Engine *engine;
    GstElement *thread, *in, *decoder, *out;

    engine = g_new0 (Engine, 1);
    engine->thread = NULL;

    in = gst_element_factory_make ("gnomevfssrc", "in");
    // we need error handleing
    // this makes memory leak
    g_assert (GST_IS_ELEMENT (in));

    /* "spider" is universal plugin wrapper for any file */
    decoder = gst_element_factory_make ("spider", "decoder");
    g_assert (GST_IS_ELEMENT (decoder));

    /* we use esd instead of default 
    out = gst_gconf_render_bin_from_key ("default/audiosink");
    g_assert (GST_IS_ELEMENT (out));
    */

    out = gst_element_factory_make ("esdsink", "out");
    g_assert (GST_IS_ELEMENT (out));

    thread = gst_thread_new ("engine");
    g_assert (GST_IS_ELEMENT (thread));
    engine->thread = thread;

    gst_bin_add_many (GST_BIN (thread), in, decoder, out, NULL);
    gst_element_connect_many (in, decoder, out, NULL);

    g_signal_connect (G_OBJECT (in), "eos",
                      G_CALLBACK (engine_meet_end_of_stream), NULL);

    return engine;
}

void
engine_free(Engine *engine)
{
    gst_element_set_state (GST_ELEMENT (engine->thread), GST_STATE_NULL);
    gst_object_unref (GST_OBJECT (engine->thread));
    g_free (engine);
}

gboolean
engine_load_uri(Engine *engine, const gchar* text_uri)
{
    GstElement *in;
    GstElement *thread;

    g_return_val_if_fail (text_uri != NULL, FALSE);

    g_print ("URI load: %s\n", text_uri);
    thread = engine->thread;

    /* set location */
    in = gst_bin_get_by_name (GST_BIN (thread), "in");
    g_object_set (G_OBJECT(in), "location", text_uri, NULL);

    gst_element_set_state (GST_ELEMENT (thread), GST_STATE_READY);
    return TRUE;
}

void
engine_play(Engine *engine)
{
    GstElement *thread = engine->thread;
    gst_element_set_state (GST_ELEMENT (thread), GST_STATE_PLAYING);
}

void
engine_pause(Engine *engine)
{
    GstElement *thread = engine->thread;
    gst_element_set_state (GST_ELEMENT (thread), GST_STATE_PAUSED);
}

void
engine_meet_end_of_stream (GstElement *in, gpointer data)
{
    Engine *engine;
    Playlist *plist;
    const gchar *text_uri;

    engine = gramophone->engine;
    plist = gramophone->playlist;

    g_print ("End of Stream\n");

    text_uri = playlist_next (plist);

    engine_load_uri (engine, text_uri);

    /* we do play now */
    engine_play (engine);
}




// old APIs
#if 0

GstElement *
engine_new (GramophoneApplet *gramophone)
{
    GstElement *thread, *in, *decoder, *out; // *buffer

    in = gst_element_factory_make ("gnomevfssrc", "in");
    g_return_val_if_fail (GST_IS_ELEMENT (in), NULL);

    /* FIXME: queue may need.
     * but inserting @buffer into thread, making engine dumb.
     */
    //buffer = gst_element_factory_make ("queue", "queue");
    //g_return_val_if_fail (GST_IS_ELEMENT (buffer), NULL);

    /* "spider" is universal plugin wrapper for any file
     * FIXME: bug 94589
     */
    decoder = gst_element_factory_make ("spider", "decoder");
    g_return_val_if_fail (GST_IS_ELEMENT (decoder), NULL);

    out = gst_gconf_render_bin_from_key ("default/audiosink");
    g_return_val_if_fail (GST_IS_ELEMENT (out), NULL);

    thread = gst_thread_new ("engine");
    gst_bin_add_many (GST_BIN (thread), in, decoder, out, NULL);
    gst_element_connect_many (in, decoder, out, NULL);

    /* connect to 'thread', rather than 'in'? */
    g_signal_connect (G_OBJECT (out), "eos",
                      G_CALLBACK (engine_meet_end_of_stream), gramophone);

    return thread;
}

void
engine_put_fuel_in (GstElement *thread,
                    gchar *uri)
{
    GstElement *in;

    g_print ("fuel location: %s\n", uri);

    /* stop first */
    gst_element_set_state (GST_ELEMENT (thread), GST_STATE_PAUSED);

    /* reset to another 'location' */
    in = gst_bin_get_by_name (GST_BIN (thread), "in");
    g_object_set (G_OBJECT(in), "location", uri, NULL);
    /* FIXME: above line
        libgnomevfs-CRITICAL **: file gnome-vfs-uri.c: line 459 (gnome_vfs_uri_new_private): assertion `text_uri != NULL' failed
     * I cannot understand why occur only once
     * for the first calling of this engine_put_fuel_in() function.
     */

    /* play again */
    gst_element_set_state (GST_ELEMENT (thread), GST_STATE_PLAYING);
}

void
engine_meet_end_of_stream (GstElement *engine_out,
                           GramophoneApplet *gramophone)
{
    GList *list;
    GtkBoxChild *child;
    GtkScrolledWindow *scroll = NULL;
    GtkTreeView *view;
    GtkTreeModel *store;
    GtkTreePath *path;
    GtkTreeIter iter;

    /* a long journey to get GtkTreeModel */
    list = GTK_BOX (GNOME_DIALOG (gramophone->playlist)->vbox)->children;
    while (list) {
        child = (GtkBoxChild *)list->data;
        if (GTK_IS_SCROLLED_WINDOW (child->widget))
            scroll = GTK_SCROLLED_WINDOW (child->widget);
        list = g_list_next (list);
    }
    g_return_if_fail (GTK_IS_SCROLLED_WINDOW (scroll));

    /* here's a clue! */
    view = GTK_TREE_VIEW (gtk_bin_get_child (GTK_BIN (scroll)));
    g_return_if_fail (GTK_IS_TREE_VIEW (view));

    store = gtk_tree_view_get_model (view);
    path = gtk_tree_row_reference_get_path (gramophone->row);
    gtk_tree_model_get_iter (store, &iter, path);

    if (gtk_tree_model_iter_next (store, &iter)) {
        /* playlist is going on... */
        g_print ("play next on playlist.\n");

        path = gtk_tree_model_get_path (store, &iter);
        gtk_tree_view_row_activated (view, path,
                                     gtk_tree_view_get_column (view, 1));
    } else
    if (g_object_get_data (G_OBJECT (gramophone->playlist), "loop_boolean")) {
        g_print ("loop in playlist.\n");

        gtk_tree_model_get_iter_root (store, &iter);
        path = gtk_tree_model_get_path (store, &iter);
        gtk_tree_view_row_activated (view, path,
                                     gtk_tree_view_get_column (view, 1));
    } else {
        /* end of playlist */
        g_print ("fuel end up. turn off the engine.\n");

        /* reset the playing row to NULL */
        gramophone->row = NULL;

        /* turn off the engine::out, to free audio device. */
        gst_element_set_state (engine_out, GST_STATE_NULL);
    }
    gtk_tree_path_free (path);
}

#endif
